<?php

namespace jrosset\CliProgram;

use jrosset\CliProgram\CommandCall\CommandCall;
use jrosset\CliProgram\CommandCall\CommandCallsList;
use ReflectionClass;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Output\OutputInterface;
use Throwable;

/**
 * A basic command
 */
class BaseCommand extends Command {
    /**
     * Initialize the command
     *
     * @param string|null $name       The command name, Null = class name
     * @param string      ...$aliases The command aliases
     */
    public function __construct (?string $name = null, string ...$aliases) {
        if ($name === null) {
            $classShortName = (new ReflectionClass($this))->getShortName();
            $name = mb_strtolower(mb_substr($classShortName, 0, 1)) . mb_substr($classShortName, 1);
        }

        parent::__construct($name);
        $this->setAliases($aliases);
    }

    /**
     * Run a list of subcommands
     *
     * Stop à the first subcommand returning other than {@see Command::SUCCESS}
     *
     * @param CommandCallsList|CommandCall $subcommandList The subcommands to run
     * @param OutputInterface              $output         The output
     *
     * @return int The last executed subcommand return code
     *
     * @throws Throwable If an error occurs
     */
    public function runSubCommands (CommandCallsList|CommandCall $subcommandList, OutputInterface $output): int {
        if (!$subcommandList instanceof CommandCallsList) {
            $subcommandList = new CommandCallsList([$subcommandList]);
        }

        /** @var CommandCall $subcommand */
        foreach ($subcommandList as $subcommand) {
            if (($returnCode = $subcommand->run($this->getApplication(), $output)) !== Command::SUCCESS) {
                return $returnCode;
            }
        }

        return Command::SUCCESS;
    }
}